import os

from nannyml_datasets.constants import GITHUB_DATASETS_REPO
from nannyml_datasets.typing import Dataset
from nannyml_datasets.regression.typing import _Dataset


REMOTE_FILE_SOURCE = os.path.join(
    GITHUB_DATASETS_REPO, "regression", "synthetic_car_price"
)
COLUMN_MAPPING = {
    "predictions_column_name": "y_pred",
    "targets_column_name": "y_true",
    "timestamps_column_name": "timestamp",
    "continuous_feature_column_names": [
        "car_age",
        "km_driven",
        "price_new",
        "accident_count",
    ],
    "categorical_feature_column_names": [
        "door_count",
        "fuel",
        "transmission",
    ],
    "excluded_column_names": ["id"],
}


class SyntheticCarPrice(Dataset):
    """"""

    def __init__(self):
        super().__init__(
            reference_dataset=_Dataset(
                remote_file_name="reference_synthetic_car_price.parquet",
                remote_file_source=REMOTE_FILE_SOURCE,
                **COLUMN_MAPPING,
            ),
            monitoring_dataset=_Dataset(
                remote_file_name="monitoring_synthetic_car_price.parquet",
                remote_file_source=REMOTE_FILE_SOURCE,
                **COLUMN_MAPPING,
            ),
        )


synthetic_car_price = SyntheticCarPrice()



import os

from nannyml_datasets.constants import GITHUB_DATASETS_REPO
from nannyml_datasets.typing import Dataset
from nannyml_datasets.multiclass_classification.typing import _Dataset


REMOTE_FILE_SOURCE = os.path.join(
    GITHUB_DATASETS_REPO, "multiclass_classification", "satellite_imagery"
)
COLUMN_MAPPING = {
    "predictions_column_name": "y_pred",
    "predicted_probabilities_column_names": {
        "0": "pred_proba_0",
        "1": "pred_proba_1",
        "2": "pred_proba_2",
        "3": "pred_proba_3",
    },
    "targets_column_name": "y_true",
    "timestamps_column_name": "timestamp",
    "continuous_feature_column_names": [f"feature_{i+1}" for i in range(1000)],
    "categorical_feature_column_names": [],
    "excluded_column_names": [],
}


class SatelliteImageryDataset(Dataset):
    """"""

    def __init__(self):
        super().__init__(
            reference_dataset=_Dataset(
                remote_file_name="reference_image_satellite.parquet",
                remote_file_source=REMOTE_FILE_SOURCE,
                **COLUMN_MAPPING,
            ),
            monitoring_dataset=_Dataset(
                remote_file_name="analysis_image_satellite.parquet",
                remote_file_source=REMOTE_FILE_SOURCE,
                **COLUMN_MAPPING,
            ),
        )


satellite_imagery = SatelliteImageryDataset()



import os

from nannyml_datasets.constants import GITHUB_DATASETS_REPO
from nannyml_datasets.typing import Dataset
from nannyml_datasets.binary_classification.typing import _Dataset


REMOTE_FILE_SOURCE = os.path.join(
    GITHUB_DATASETS_REPO, "binary_classification", "synthetic_car_loan"
)
COLUMN_MAPPING = {
    "predictions_column_name": "y_pred",
    "predicted_probabilities_column_name": "y_pred_proba",
    "targets_column_name": "repaid",
    "timestamps_column_name": "timestamp",
    "continuous_feature_column_names": [
        "car_value",
        "debt_to_income_ratio",
        "driver_tenure",
        "loan_length",
    ],
    "categorical_feature_column_names": [
        "repaid_loan_on_prev_car",
        "salary_range",
        "size_of_downpayment",
    ],
    "excluded_column_names": ["id"],
}


class SyntheticCarLoanDataset(Dataset):
    """"""

    def __init__(self):
        super().__init__(
            reference_dataset=_Dataset(
                remote_file_name="reference_synthetic_car_loan.parquet",
                remote_file_source=REMOTE_FILE_SOURCE,
                **COLUMN_MAPPING,
            ),
            monitoring_dataset=_Dataset(
                remote_file_name="monitoring_synthetic_car_loan.parquet",
                remote_file_source=REMOTE_FILE_SOURCE,
                **COLUMN_MAPPING,
            ),
        )


synthetic_car_loan = SyntheticCarLoanDataset()



import os

from nannyml_datasets.constants import GITHUB_DATASETS_REPO
from nannyml_datasets.typing import Dataset
from nannyml_datasets.binary_classification.typing import _Dataset


REMOTE_FILE_SOURCE = os.path.join(
    GITHUB_DATASETS_REPO, "binary_classification", "hotel_booking"
)
COLUMN_MAPPING = {
    "predictions_column_name": "y_pred",
    "predicted_probabilities_column_name": "y_pred_proba",
    "targets_column_name": "is_canceled",
    "timestamps_column_name": "timestamp",
    "continuous_feature_column_names": [
        "adr",
        "adults",
        "babies",
        "booking_changes",
        "children",
        "days_in_waiting_list",
        "lead_time",
        "parking_spaces",
        "previous_bookings_not_canceled",
        "previous_cancellations",
        "stays_in_weekend_nights",
        "stays_in_week_nights",
        "total_of_special_requests",
    ],
    "categorical_feature_column_names": [
        "agent",
        "assigned_room_type",
        "company",
        "country",
        "customer_type",
        "deposit_type",
        "distribution_channel",
        "hotel",
        "market_segment",
        "meal",
        "reserved_room_type",
    ],
    "excluded_column_names": ["index"],
}


class HotelBookingDataset(Dataset):
    """"""

    def __init__(self):
        super().__init__(
            reference_dataset=_Dataset(
                remote_file_name="reference_hotel_booking.parquet",
                remote_file_source=REMOTE_FILE_SOURCE,
                **COLUMN_MAPPING,
            ),
            monitoring_dataset=_Dataset(
                remote_file_name="monitoring_hotel_booking.parquet",
                remote_file_source=REMOTE_FILE_SOURCE,
                **COLUMN_MAPPING,
            ),
        )


hotel_booking = HotelBookingDataset()



import os

from nannyml_datasets.constants import GITHUB_DATASETS_REPO
from nannyml_datasets.typing import Dataset
from nannyml_datasets.multiclass_classification.typing import _Dataset


REMOTE_FILE_SOURCE = os.path.join(
    GITHUB_DATASETS_REPO, "multiclass_classification", "synthetic_credit_card"
)
COLUMN_MAPPING = {
    "predictions_column_name": "y_pred",
    "predicted_probabilities_column_names": {
        "prepaid_card": "y_pred_proba_prepaid_card",
        "upmarket_card": "y_pred_proba_upmarket_card",
        "highstreet_card": "y_pred_proba_highstreet_card",
    },
    "targets_column_name": "y_true",
    "timestamps_column_name": "timestamp",
    "continuous_feature_column_names": [
        "app_behavioral_score",
        "requested_credit_limit",
        "credit_bureau_score",
        "stated_income",
    ],
    "categorical_feature_column_names": [
        "acq_channel",
        "app_channel",
        "is_customer",
    ],
    "excluded_column_names": ["id"],
}


class SyntheticCreditCard(Dataset):
    """"""

    def __init__(self):
        super().__init__(
            reference_dataset=_Dataset(
                remote_file_name="reference_synthetic_credit_card.parquet",
                remote_file_source=REMOTE_FILE_SOURCE,
                **COLUMN_MAPPING,
            ),
            monitoring_dataset=_Dataset(
                remote_file_name="monitoring_synthetic_credit_card.parquet",
                remote_file_source=REMOTE_FILE_SOURCE,
                **COLUMN_MAPPING,
            ),
        )


synthetic_credit_card = SyntheticCreditCard()



from __future__ import annotations
import os
import tempfile
import time
import pyarrow as pa
import pyarrow.parquet as pq
import numpy.typing as npt
import requests

from typing import Iterable, List, Mapping, Optional


class _Dataset:
    def __init__(
        self,
        remote_file_name: str,
        remote_file_source: str,
        continuous_feature_column_names: Optional[List[str]] = None,
        categorical_feature_column_names: Optional[List[str]] = None,
        remote_file_checksum: Optional[str] = None,
        predictions_column_name: Optional[str] = None,
        targets_column_name: Optional[str] = None,
        timestamps_column_name: Optional[str] = None,
        excluded_column_names: Optional[List[str]] = None,
    ):
        self.remote_file_name = remote_file_name
        self.remote_file_source = remote_file_source
        self.remote_file_checksum = remote_file_checksum
        self.continuous_feature_column_names = continuous_feature_column_names or []
        self.categorical_feature_column_names = categorical_feature_column_names or []
        self.predictions_column_name = predictions_column_name
        self.targets_column_name = targets_column_name
        self.timestamps_column_name = timestamps_column_name
        self.excluded_column_names = excluded_column_names or []

        self._data: Optional[pa.Table] = None

    @property
    def data(self) -> pa.Table:
        if self._data is None:
            with tempfile.NamedTemporaryFile() as f:
                parquet_path = download_source(
                    f,
                    self.remote_file_source,
                    self.remote_file_name,
                    self.remote_file_checksum,
                )
                self._data = pq.read_table(parquet_path)

        return self._data

    @property
    def predictions(self) -> npt.NDArray:
        assert (
            self.predictions_column_name is not None
        ), "No predictions column name provided"
        return self.data[self.predictions_column_name].to_numpy()

    @property
    def targets(self) -> npt.NDArray:
        assert self.targets_column_name is not None, "No targets column name provided"
        return self.data[self.targets_column_name].to_numpy()

    @property
    def timestamps(self) -> npt.NDArray:
        assert (
            self.timestamps_column_name is not None
        ), "No timestamps column name provided"
        return self.data[self.timestamps_column_name].to_numpy()

    @property
    def categorical_features(self) -> Iterable[npt.NDArray]:
        return (
            self.data[col].to_numpy() for col in self.categorical_feature_column_names
        )

    @property
    def continuous_features(self) -> Iterable[npt.NDArray]:
        return (
            self.data[col].to_numpy() for col in self.continuous_feature_column_names
        )

    @property
    def features(self) -> Mapping[str, npt.NDArray]:
        return {
            col: self.data[col].to_numpy()
            for col in self.categorical_feature_column_names
            + self.continuous_feature_column_names
        }


class Dataset:
    def __init__(
        self,
        reference_dataset: _Dataset,
        monitoring_dataset: _Dataset,
    ) -> None:
        self.reference_dataset = reference_dataset
        self.monitoring_dataset = monitoring_dataset

    @property
    def reference(self) -> _Dataset:
        return self.reference_dataset

    @property
    def monitoring(self) -> _Dataset:
        return self.monitoring_dataset


def download_source(
    local_file,
    remote_file_source: str,
    remote_file_name: str,
    remote_file_checksum: Optional[str] = None,
    n_retries: int = 3,
    delay: int = 3,
):
    # Perform download
    try:
        with requests.get(
            os.path.join(remote_file_source, remote_file_name)
        ) as response:
            response.raise_for_status()
            local_file.write(response.content)
    except TimeoutError:
        if n_retries == 0:
            raise

        n_retries -= 1
        time.sleep(delay)

    # Checksum verification

    return local_file.name



from typing import Dict, List, Optional
import numpy.typing as npt
from nannyml_datasets.typing import _Dataset as _BaseDataset


ClassLabel = str


class _Dataset(_BaseDataset):
    def __init__(
        self,
        remote_file_name: str,
        remote_file_source: str,
        continuous_feature_column_names: Optional[List[str]] = None,
        categorical_feature_column_names: Optional[List[str]] = None,
        remote_file_checksum: Optional[str] = None,
        predicted_probabilities_column_names: Optional[Dict[ClassLabel, str]] = None,
        predictions_column_name: Optional[str] = None,
        targets_column_name: Optional[str] = None,
        timestamps_column_name: Optional[str] = None,
        excluded_column_names: Optional[List[str]] = None,
    ):
        super().__init__(
            remote_file_name,
            remote_file_source,
            continuous_feature_column_names,
            categorical_feature_column_names,
            remote_file_checksum,
            predictions_column_name,
            targets_column_name,
            timestamps_column_name,
            excluded_column_names,
        )

        self.predicted_probabilities_column_names = predicted_probabilities_column_names

    @property
    def predicted_probabilities(self) -> Dict[ClassLabel, npt.NDArray]:
        return {
            class_label: self.data[column_name].to_numpy()
            for class_label, column_name in self.predicted_probabilities_column_names.items()
        }

    @property
    def classes(self) -> List[ClassLabel]:
        return list(sorted(self.predicted_probabilities_column_names.keys()))
